import requests


class HttpHandlerError(Exception):
    def __init__(self, r: requests.Response):
        super().__init__(f"post log error, url={r.url}, status_code={r.status_code}, text={r.text}")



import re
from typing import List
from datetime import datetime

from logkk.level import Level
from logkk.handlers import Handler, StreamHandler


class Logger(object):
    """
    记录日志的实际操作者，可以理解为worker
    """

    def __init__(self, name, level=Level.INFO, fmt=None, handlers=None):
        self.name = name
        self.level = level
        self.fmt = fmt
        self.handlers: List[Handler] = handlers if handlers else []
        self._datetime_fmt = "%Y-%m-%d %H:%M:%S"
        self._default_handler = StreamHandler()

    def _get_content(self, level, *args, **kwargs):
        """组装日志内容"""
        keys = re.findall(r"{(\w+)}", self.fmt)
        data = dict()
        for key in keys:
            if key == "message":
                data[key] = self._get_message(*args, **kwargs)
                continue
            if key == "level":
                data[key] = level.upper()
                continue
            data[key] = getattr(self, f"_get_{key}")()
        content = self.fmt.format(**data)
        return content

    def _get_name(self):
        return self.name

    def _get_datetime(self):
        return datetime.now().strftime(self._datetime_fmt)

    @staticmethod
    def _get_message(*args, **kwargs):
        message = " ".join([str(arg) for arg in args])
        for key, val in kwargs.items():
            message += f" {key}={val}"
        return message

    def _write(self, level, *args, **kwargs):
        content = self._get_content(level, *args, **kwargs)
        if self.handlers:
            for handler in self.handlers:
                handler.write(content)
        else:
            self._default_handler.write(content)

    def debug(self, *args, **kwargs):
        if self.level > Level.DEBUG:
            return
        self._write("debug", *args, **kwargs)

    def info(self, *args, **kwargs):
        if self.level > Level.INFO:
            return
        self._write("info", *args, **kwargs)

    def warn(self, *args, **kwargs):
        if self.level > Level.WARNING:
            return
        self._write("warn", *args, **kwargs)

    def error(self, *args, **kwargs):
        if self.level > Level.ERROR:
            return
        self._write("error", *args, **kwargs)



import threading
from abc import ABC

file_lock = threading.Lock()
stream_lock = threading.Lock()


class Handler(ABC):

    def write(self, content):
        pass


class StreamHandler(Handler):

    def write(self, content):
        with stream_lock:
            print(content, flush=True)


class FileHandler(Handler):

    def __init__(self, filepath, encoding="utf8"):
        self.filepath = filepath
        self.encoding = encoding
        self.file_obj = None

    def write(self, content):
        with file_lock:
            if not self.file_obj:
                self.file_obj = open(file=self.filepath,
                                     mode="a+",
                                     encoding=self.encoding)
            self.file_obj.write(content + "\n")
            self.file_obj.flush()

    def close(self):
        self.file_obj.close()



from logkk.level import Level
from logkk.logger import Logger
from logkk.handlers import Handler

DEFAULT_FORMAT = "[{datetime}] [{level}] [{name}] {message}"


class LogManager(object):

    def __init__(self,
                 name="logcc",
                 level=Level.INFO,
                 fmt=DEFAULT_FORMAT,
                 handlers=None):
        self.name = name
        self.level = level
        self.fmt = fmt
        self.handlers = handlers or []
        self._default_logger = self.get_logger(self.name)

    def set_format(self, fmt: str):
        self.fmt = fmt

    def set_level(self, level: int):
        self.level = level

    def add_handler(self, handler: Handler):
        self.handlers.append(handler)

    def get_logger(self, name=None) -> Logger:
        """
        :param name: 模块名称或者文件名称
        :return: Logger对象
        """
        return Logger(name=name or self.name,
                      level=self.level,
                      fmt=self.fmt,
                      handlers=self.handlers)

    def debug(self, *args, **kwargs):
        self._default_logger.info(*args, **kwargs)

    def info(self, *args, **kwargs):
        self._default_logger.info(*args, **kwargs)

    def warn(self, *args, **kwargs):
        self._default_logger.info(*args, **kwargs)

    def error(self, *args, **kwargs):
        self._default_logger.info(*args, **kwargs)


if __name__ == "__main__":
    log_manager = LogManager()
    log_manager.info("this is a info log")
    log_manager.warn("this is a warn log")
    logger = log_manager.get_logger(name="hello")
    logger.info("this is a info log")
    logger.warn("this is a warn log")



import setuptools

with open("README.md", "r", encoding="utf8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="logkk",  # 模块名称
    version="0.5.0",  # 当前版本
    author="olivetree",  # 作者
    author_email="olivetree123@163.com",  # 作者邮箱
    description="日志组件",  # 模块简介
    long_description=long_description,  # 模块详细介绍
    long_description_content_type="text/markdown",  # 模块详细介绍格式
    url="https://github.com/olivetree/logkk",  # 模块github地址
    packages=["logkk"],  # 自动找到项目中导入的模块
    # 模块相关的元数据（更多的描述）
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # 依赖模块
    install_requires=[],
    # python版本
    python_requires=">=3.7",
)
